/*****************************************************************************
 *
 * wolcub, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ebook@ndl.kiev.ua>
 *
 * Heavily based on BITMAP OEM DLL example from WDK by Microsoft DDK Team.
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "precomp.h"
#include "magick/MagickCore.h"
#include "wolutil.h"

const DWORD IMG_WIDTH = 600, IMG_HEIGHT = 800;
double OVERLAP_RATIO = 0.05;

void FlushImage(Image* image, WOLWriter* pWolWriter)
{
    ExceptionInfo ex;
    DWORD width = image->columns, height = image->rows;

    lUInt8* pixels = new lUInt8[width * height];
    ExportImagePixels(image, 0, 0, width, height, "I", CharPixel, pixels, &ex);

    // Currently lBook V3 supports only 4-level grayscale
    // or monochrome, but we work in 4-level grayscale only.
    DWORD dwPixelsCount = width * height;

    lUInt8* buf = new lUInt8[dwPixelsCount / 4];
    memset(buf, 0, dwPixelsCount / 4);

    for (DWORD i = 0; i < dwPixelsCount; ++i)
        buf[i / 4] |= (((unsigned short)pixels[i] + 42) / 85) << (6 - 2 * (i % 4));

    pWolWriter->addImage(width, height, buf, 2);

    delete[] buf;
    delete[] pixels;
}

void ProcessImage(lUInt8* bitmap, DWORD& dwBufSize, DWORD& dwNewImgHeight,
                  DWORD width, DWORD height, BYTE btBPP,
                  BOOL bLandscape, WOLWriter* pWolWriter, BOOL bFlush)
{
    DWORD orig_width = width, orig_height = height;
    lUInt8* bmp_orig = bitmap;
    lUInt8* bmp_new = NULL;

    if (btBPP == 1)
    {
        bmp_new = new lUInt8[width * height];

        for (int y = 0; y < height; ++y)
            for (int x = 0; x < width; ++x)
            {
                DWORD index = x + y * width;
                bmp_new[index] = (bitmap[index / 8] & (1 << (7 - index % 8)))? 255: 0;
            }
        bitmap = bmp_new;
    }

    ExceptionInfo ex;
    Image* image = ConstituteImage(width, height, "I", CharPixel, bitmap, &ex);

    if (bLandscape)
    {
        height = height*IMG_HEIGHT/width;
        width = IMG_HEIGHT;
        Image* img_res = ResizeImage(image, width, height, LanczosFilter, 1.0, &ex);
        DestroyImage(image);
        image = img_res;

        while (height >= IMG_WIDTH || bFlush && height > 0)
        {
            RectangleInfo rect;
            rect.x = 0;
            rect.y = image->rows - height;
            rect.width = IMG_HEIGHT;
            rect.height = IMG_WIDTH;
            Image* crop = CropImage(image, &rect, &ex);
            Image* image_rot = RotateImage(crop, 90, &ex);
            DestroyImage(crop);
            PosterizeImage(image_rot, 4, MagickTrue);
            if (image_rot->columns < IMG_WIDTH)
            {
                rect.x = 0;
                rect.y = 0;
                rect.width = IMG_WIDTH;
                rect.height = IMG_HEIGHT;
                Image* img_ext = ExtentImage(image_rot, &rect, &ex);
                DestroyImage(image_rot);
                image_rot = img_ext;
            }
            FlushImage(image_rot, pWolWriter);
            DestroyImage(image_rot);

            if (height <= IMG_WIDTH)
                height = 0;
            else
                height -= IMG_WIDTH * (1.0 - OVERLAP_RATIO);
        }

        DestroyImage(image);

        DWORD row = (image->rows - height) * orig_width/IMG_HEIGHT;
        dwBufSize = (orig_height - row) * orig_width * btBPP / 8;
        MoveMemory(bmp_orig, bmp_orig + orig_width * row * btBPP / 8, dwBufSize);
        dwNewImgHeight = orig_height - row;
    }
    else
    {
        height = height*IMG_WIDTH/width;
        width = IMG_WIDTH;
        Image* img_res = ResizeImage(image, width, height, LanczosFilter, 1.0, &ex);
        DestroyImage(image);
        image = img_res;
        PosterizeImage(image, 4, MagickTrue);
        FlushImage(image, pWolWriter);
        DestroyImage(image);
        dwBufSize = 0;
        dwNewImgHeight = 0;
    }

    if (bmp_new)
        delete[] bmp_new;
}
